{ ----------------------------------------------------------------------------}
{ Data aware components for Delphi.                                           }
{ Copyright 1995, Curtis White.  All Rights Reserved.                         }
{ These components can be freely used and distributed in commercial and       }
{ private environments, provied this notice is not modified in any way.       }
{ ----------------------------------------------------------------------------}
{ Feel free to contact me if you have any questions, comments or suggestions  }
{ at cwhite@teleport.com                                                      }
{ ----------------------------------------------------------------------------}
{ Date last modified:  05/05/95                                               }
{ ----------------------------------------------------------------------------}
{ ----------------------------------------------------------------------------}
{ TDBButton v1.00                                                             }
{ ----------------------------------------------------------------------------}
{ Description:                                                                }
{   A data aware button.                                                      }
{ Features:                                                                   }
{   Same as a normal button except the caption can be set by a database field.}
{ ----------------------------------------------------------------------------}
{ ----------------------------------------------------------------------------}
{ Revision History:                                                           }
{ 1.00:  Initial release                                                      }
{ ----------------------------------------------------------------------------}

{ ----------------------------------------------------------------------------}
{ TDBBitBtn v1.00                                                             }
{ ----------------------------------------------------------------------------}
{ Description:                                                                }
{   A data aware bitmap button.                                               }
{ Features:                                                                   }
{   Same as a normal bitmap button except the caption can be set by a database}
{   field.}
{ ----------------------------------------------------------------------------}
{ ----------------------------------------------------------------------------}
{ Revision History:                                                           }
{ 1.00:  Initial release                                                      }
{ ----------------------------------------------------------------------------}

{ ----------------------------------------------------------------------------}
{ TDBRadioButton v1.00                                                        }
{ ----------------------------------------------------------------------------}
{ Description:                                                                }
{   A data aware radio button.                                                }
{ Features:                                                                   }
{   Same as a normal radio button except the caption can be set by a database }
{   field.}
{ ----------------------------------------------------------------------------}
{ ----------------------------------------------------------------------------}
{ Revision History:                                                           }
{ 1.00:  Initial release                                                      }
{ ----------------------------------------------------------------------------}

{ ----------------------------------------------------------------------------}
{ TDBLabel3D v1.00                                                            }
{ ----------------------------------------------------------------------------}
{ Description:                                                                }
{   A data aware 3-D label.                                                   }
{ Features:                                                                   }
{   The label style can be set to Raised, Lowered, or None.                   }
{   The label caption can be set by a database field.                         }
{ ----------------------------------------------------------------------------}
{ ----------------------------------------------------------------------------}
{ Revision History:                                                           }
{ 1.00:  Initial release                                                      }
{ ----------------------------------------------------------------------------}

{ ----------------------------------------------------------------------------}
{ TDBMaskEdit v1.00                                                           }
{ ----------------------------------------------------------------------------}
{ Description:                                                                }
{   A data aware masked edit box.                                             }
{ Features:                                                                   }
{   Same as a normal masked edit box except box can be filled by a database   }
{     field.                                                                  }
{ ----------------------------------------------------------------------------}
{ ----------------------------------------------------------------------------}
{ Revision History:                                                           }
{ 1.00:  Initial release                                                      }
{ ----------------------------------------------------------------------------}

unit Dabtns;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, DB, DBCtrls, DBTables, Buttons, Mask;

type
  { TDBButton }
  { Data Aware Button }
  TDBButton = class(TButton)
  private
    { Variables for properties }
    FDataLink: TFieldDataLink;

    { Procedures for setting property values }
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure DataChange(Sender: TObject);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Field in the database table }
    property DataField: string read GetDataField write SetDataField;
    { Datasource attached to the database table }
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;


  { TDBBitBtn }
  { Data Aware BitBtn }
  TDBBitBtn = class(TBitBtn)
  private
    { Variables for properties }
    FDataLink: TFieldDataLink;

    { Procedures for setting property values }
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure DataChange(Sender: TObject);
  protected
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Field in the database table }
    property DataField: string read GetDataField write SetDataField;
    { Datasource attached to the database table }
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;


  { TDBRadioButton }
  { Data Aware Radio Button }
  TDBRadioButton = class(TRadioButton)
  private
    { Variables for properties }
    FDataLink: TFieldDataLink;

    { Procedures for setting property values }
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure DataChange(Sender: TObject);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Field in the database table }
    property DataField: string read GetDataField write SetDataField;
    { Datasource attached to the database table }
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;


  { TDBLabel3D }
  { Data Aware 3-D Label }
  { Style of 3-D label }
  TTextStyle = ( tsNone, tsRaised, tsLowered );

  TDBLabel3D = class(TLabel)
  private
    { Variables for properties }
    FTextStyle: TTextStyle;
    FDataLink: TFieldDataLink;

    { Procedures for setting property values }
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure DataChange(Sender: TObject);

    { Procedure to draw the 3-D label }
    procedure DrawLabel3D(var TextRect: TRect; SpecialFlags: Word);
  protected
    procedure Paint; override;
    { Procedure to set the text style }
    procedure SetTextStyle( Value : TTextStyle );
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Style of the 3-D label }
    property TextStyle: TTextStyle read FTextStyle write SetTextStyle
        default tsLowered;
    { Field in the database table }
    property DataField: string read GetDataField write SetDataField;
    { Datasource attached to the database table }
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;


  { TDBMaskEdit }
  { Data Aware Mask Edit }
  TDBMaskEdit = class(TDBEdit)
  published
    { Publish Mask and Text properties }
    property EditMask;
    property Text;
  end;


procedure Register;

implementation

{ TDBButton }
{ Data Aware Button }
{ Override the constructor to initialize variables }
constructor TDBButton.Create(AOwner: TComponent);
begin
  { Inherit original constructor }
  inherited Create(AOwner);
  { Add new initializations }
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange;
end;

{ Override the destructor to destroy variables }
destructor TDBButton.Destroy;
begin
  FDataLink.OnDataChange := nil;
  FDataLink.Free;
  inherited Destroy;
end;

{ Get database field }
function TDBButton.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

{ Get datasource }
function TDBButton.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{ Set database field }
procedure TDBButton.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

{ Set datasource }
procedure TDBButton.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

{ Set caption when data changes }
procedure TDBButton.DataChange(Sender: TObject);
begin
  if FDataLink.Field = nil then
    Caption := ''
  else
    Caption := FDataLink.Field.AsString;
end;

{ ----------------------------------------------------------------------------}

{ TDBBitBtn }
{ Data Aware BitBtn }
{ Override the constructor to initialize variables }
constructor TDBBitBtn.Create(AOwner: TComponent);
begin
  { Inherit original constructor }
  inherited Create(AOwner);
  { Add new initializations }
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange;
end;

{ Override the destructor to destroy variables }
destructor TDBBitBtn.Destroy;
begin
  FDataLink.OnDataChange := nil;
  FDataLink.Free;
  inherited Destroy;
end;

{ Get database field }
function TDBBitBtn.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

{ Get datasource }
function TDBBitBtn.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{ Set database field }
procedure TDBBitBtn.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

{ Set datasource }
procedure TDBBitBtn.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

{ Set caption when data changes }
procedure TDBBitBtn.DataChange(Sender: TObject);
begin
  if FDataLink.Field = nil then
    Caption := ''
  else
    Caption := FDataLink.Field.AsString;
end;

{ ----------------------------------------------------------------------------}

{ TDBRadioButton }
{ Data Aware Radio Button }
{ Override the constructor to initialize variables }
constructor TDBRadioButton.Create(AOwner: TComponent);
begin
  { Inherit original constructor }
  inherited Create(AOwner);
  { Add new initializations }
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange;
end;

{ Override the destructor to destroy variables }
destructor TDBRadioButton.Destroy;
begin
  FDataLink.OnDataChange := nil;
  FDataLink.Free;
  inherited Destroy;
end;

{ Get database field }
function TDBRadioButton.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

{ Get datasource }
function TDBRadioButton.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{ Set database field }
procedure TDBRadioButton.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

{ Set datasource }
procedure TDBRadioButton.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

{ Set caption when data changes }
procedure TDBRadioButton.DataChange(Sender: TObject);
begin
  if FDataLink.Field = nil then
    Caption := ''
  else
    Caption := FDataLink.Field.AsString;
end;

{ ----------------------------------------------------------------------------}

{ TDBLabel3D }
{ Data Aware 3D Label }
{ Override the constructor to initialize variables }
constructor TDBLabel3D.Create(AOwner: TComponent);
begin
  { Inherit original constructor }
  inherited Create(AOwner);
  { Add new initializations }
  FTextStyle := tsLowered;
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange;
end;

{ Override the destructor to destroy variables }
destructor TDBLabel3D.Destroy;
begin
  FDataLink.OnDataChange := nil;
  FDataLink.Free;
  inherited Destroy;
end;

{ Set the 3-D style of the label }
procedure TDBLabel3D.SetTextStyle( Value : TTextStyle );
begin
  if Value <> FTextStyle then
  begin
    FTextStyle := Value;
    Invalidate;
  end;
end;

{ Draw the text and the 3-D bevel }
procedure TDBLabel3D.DrawLabel3D( var TextRect : TRect; SpecialFlags : Word );
var
  Text        : array[ 0..255 ] of Char;
  WorkRect    : TRect;
  TopColor    : TColor;
  BottomColor : TColor;
begin
  { Copy the text to a buffer }
  GetTextBuf(Text, SizeOf(Text));

  { Process special characters }
  if (SpecialFlags and DT_CALCRECT <> 0) and
     ((Text[0] = #0) or ShowAccelChar and
      (Text[0] = '&') and
      (Text[1] = #0)) then
    StrCopy(Text, ' ');

  if not ShowAccelChar then
    SpecialFlags := SpecialFlags or DT_NOPREFIX;

  { Set the font }
  Canvas.Font := Font;

  { Set highlight and shadow colors }
  TopColor := clBtnHighlight;
  BottomColor := clBtnShadow;

  { If the text style is lowered then reverse the highlight and shadow colors }
  if FTextStyle = tsLowered then
  begin
    TopColor := clBtnShadow;
    BottomColor := clBtnHighlight;
  end;

  { If the text style is lowered or raised then draw it with its bevel }
  if FTextStyle in [ tsLowered, tsRaised ] then
  begin
    { Draw the bottom color }
    { Set the working rectangle coordinates }
    WorkRect := TextRect;
    { Offset it by 1 pixel }
    OffsetRect( WorkRect, 1, 1 );
    { Set color to the bottom color }
    Canvas.Font.Color := BottomColor;
    { Draw the text }
    DrawText(Canvas.Handle, Text, StrLen(Text), WorkRect, SpecialFlags);

    { Draw the top color }
    { Set the working rectangle coordinates }
    WorkRect := TextRect;
    { Offset it by -1 pixel }
    OffsetRect( WorkRect, -1, -1 );
    { Set color to the top color }
    Canvas.Font.Color := TopColor;
    { Draw the text }
    DrawText(Canvas.Handle, Text, StrLen(Text), WorkRect, SpecialFlags);
  end;

  { Set the actual font color }
  Canvas.Font.Color := Font.Color;
  { If it is disabled then set the font color to gray }
  if not Enabled then
    Canvas.Font.Color := clGrayText;
  { Draw the text }
  DrawText(Canvas.Handle, Text, StrLen(Text), TextRect, SpecialFlags);
end;

{ Paint 3-D Label }
procedure TDBLabel3D.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Rect: TRect;
begin
  with Canvas do
  begin
    { If transparent property is off then draw background }
    if not Transparent then
    begin
      { Set brush color and style then paint the background }
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;

    { Set the brush style and text rectangle coordinates then paint the text }
    Brush.Style := bsClear;
    Rect := ClientRect;
    DrawLabel3D(Rect, (DT_EXPANDTABS or DT_WORDBREAK ) or
                Alignments[Alignment]);
  end;
end;

{ Get database field }
function TDBLabel3D.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

{ Get datasource }
function TDBLabel3D.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{ Set database field }
procedure TDBLabel3D.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

{ Set datasource }
procedure TDBLabel3D.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

{ Set caption when data changes }
procedure TDBLabel3D.DataChange(Sender: TObject);
begin
  if FDataLink.Field = nil then
    Caption := ''
  else
    Caption := FDataLink.Field.AsString;
end;

{ ----------------------------------------------------------------------------}

{ Register the components }
procedure Register;
begin
  RegisterComponents('Data Controls', [TDBButton, TDBBitBtn, TDBRadioButton,
                                       TDBLabel3D, TDBMaskEdit]);
end;

end.
