unit Sfingr;


{   Module:     SFngr
    Author:     Rick Lewis
    Date:       Apr-95
    Copyright:  Lenham Computer Services Ltd 1995
    Description: Simple Interface Finger client Example for IpSocket

    The Information and code provided hereunder (collectively referred to
    as "SOFTWARE") is provided as is without warranty of any kind, either
    express or implied, including but not limited to the implied warranties
    of merchanantability and fitness for a particular purpose. In no event
    shall Lenham Computer Services Ltd be liable for any damages whatsoever
    including direct, indirect, incidental, consequential, loss of business
    profits or special damages, even if Lenham Computer Services Ltd have
    been advised of the possibility of such damages.
}


interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, IpSocket;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Edit1: TEdit;
    Button1: TButton;
    Memo1: TMemo;
    StatusBar: TPanel;
    IpSocket1: TIpSocket;
    procedure Button1Click(Sender: TObject);
    procedure IPSocket1DataAvailable(Socket: Word);
    procedure IPSocket1SocketClose(Socket: Word);
    procedure IPSocket1TimeOut(Socket: Word);
    procedure IPSocket1SocketConnect(Socket: Word);
    procedure IPSocket1DataReady(Socket: Word);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  DataBack: TMemoryStream;
  UserName, HostName : String;

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
var
    iPos : Integer;
begin
    { User has entered name@Host to finger }
    if Edit1.Text <> '' then
    begin
        iPos := Pos('@',Edit1.Text);
        if iPos > 0 Then
        begin
            Button1.Enabled := False;
            Inc(iPos);
            Username := Copy(Edit1.Text, 1, iPos-2);
            HostName := Copy(Edit1.Text, iPos, Length(Edit1.Text));
            StatusBar.Caption := 'Connecting... (' + HostName +')';
            DataBack.Clear;
            Memo1.Clear;
            { Connect to Remote 'finger' port }
            IpSocket1.RemoteAddress := HostName;
            IpSocket1.RemotePort := 'finger';
            IpSocket1.ConnectToRemote;
        end
        else
            MessageDlg('No Local Host names allowed' + #13#10 +
            'Format is name@host ', mtInformation, [mbOK], 0);
    end
    else
        MessageDlg('Please supply a person to finger' + #13#10 +
        'Format is person@host ', mtInformation, [mbOK], 0);
end;

procedure TForm1.IpSocket1DataAvailable(Socket: Word);
var
    Data : TStringList;
begin
    { Stream all returned data to a memory stream until disconnected }
    Data := TStringList.Create;
    IpSocket1.GetFromRemote(Data);
    DataBack.Write(Data.GetText^,StrLen(Data.GetText));
    Data.Free;
end;

procedure TForm1.IpSocket1SocketClose(Socket: Word);
const
    Null : word = $0000;
begin
    { Remote socket has closed, close down our connection }
    DataBack.Write(Null,2);  { Add null termination byte }
    Memo1.SetTextBuf(DataBack.Memory);
    StatusBar.Caption := 'Disconnected.';
    IpSocket1.DisconnectFromRemote;
    Button1.Enabled := True;
end;

procedure TForm1.IpSocket1TimeOut(Socket: Word);
begin
    { Failed to connect to remote so do a disconnect }
    StatusBar.Caption := 'Failed to Connect to ' + HostName;
    IpSocket1.DisconnectFromRemote;
    Button1.Enabled := True;
end;

procedure TForm1.IpSocket1SocketConnect(Socket: Word);
begin
    { Got a connection message let the user known about it }
    StatusBar.Caption := 'Connected to ' + HostName;
end;


procedure TForm1.IpSocket1DataReady(Socket: Word);
Var
    Data: TStringList;
begin
    { ready to send data - send username string }
    Data := TStringList.Create;
    Data.Add(Username);
    IpSocket1.SendToRemote(Data);
    Data.Free;
end;

{ form create/destroy = create/destroy memory stream }

procedure TForm1.FormCreate(Sender: TObject);
begin
    DataBack := TMemoryStream.Create;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
    DataBack.Free;
end;

end.
